@@ -78,28 +78,6 @@ public:
 		WFFuture<void> future;
 	};
 
-public:
-	class ReplyGuard
-	{
-	public:
-		ReplyGuard(SubTask *server_task)
-		{
-			SeriesWork *series = series_of(server_task);
-			assert(series);
-			assert(server_task == series->get_last_task());
-			this->cond = WFTaskFactory::create_conditional(server_task);
-			series->set_last_task(this->cond);
-		}
-
-		~ReplyGuard()
-		{
-			this->cond->signal(NULL);
-		}
-
-	private:
-		WFConditional *cond;
-	};
-
 private:
 	static void __timer_future_callback(WFTimerTask *task);
 	static void __fio_future_callback(WFFileIOTask *task);
