@@ -77,7 +77,7 @@ private:
 };
 
 /* To support TLS SNI. */
-class RouteTargetSNI : public RouteManager::RouteTarget
+class RouteTargetTCPSNI : public RouteTargetTCP
 {
 private:
 	virtual int init_ssl(SSL *ssl)
@@ -92,7 +92,27 @@ private:
 	std::string hostname;
 
 public:
-	RouteTargetSNI(const std::string& name) : hostname(name)
+	RouteTargetTCPSNI(const std::string& name) : hostname(name)
+	{
+	}
+};
+
+class RouteTargetSCTPSNI : public RouteTargetSCTP
+{
+private:
+	virtual int init_ssl(SSL *ssl)
+	{
+		if (SSL_set_tlsext_host_name(ssl, this->hostname.c_str()) > 0)
+			return 0;
+		else
+			return -1;
+	}
+
+private:
+	std::string hostname;
+
+public:
+	RouteTargetSCTPSNI(const std::string& name) : hostname(name)
 	{
 	}
 };
@@ -121,7 +141,7 @@ public:
 	CommSchedObject *request_object;
 	CommSchedGroup *group;
 	std::mutex mutex;
-	std::vector<CommSchedTarget *> targets;
+	std::vector<RouteManager::RouteTarget *> targets;
 	struct list_head breaker_list;
 	uint64_t key;
 	int nleft;
@@ -140,34 +160,35 @@ public:
 	int init(const struct RouteParams *params);
 	void deinit();
 
-	void notify_unavailable(CommSchedTarget *target);
-	void notify_available(CommSchedTarget *target);
+	void notify_unavailable(RouteManager::RouteTarget *target);
+	void notify_available(RouteManager::RouteTarget *target);
 	void check_breaker();
 
 private:
 	void free_list();
-	CommSchedTarget *create_target(const struct RouteParams *params,
-								   const struct addrinfo *addrinfo);
+	RouteManager::RouteTarget *create_target(const struct RouteParams *params,
+											 const struct addrinfo *addrinfo);
 	int add_group_targets(const struct RouteParams *params);
 };
 
 struct __breaker_node
 {
-	CommSchedTarget *target;
+	RouteManager::RouteTarget *target;
 	int64_t timeout;
 	struct list_head breaker_list;
 };
 
-CommSchedTarget *RouteResultEntry::create_target(const struct RouteParams *params,
-												 const struct addrinfo *addr)
+RouteManager::RouteTarget *
+RouteResultEntry::create_target(const struct RouteParams *params,
+								const struct addrinfo *addr)
 {
-	CommSchedTarget *target;
+	RouteManager::RouteTarget *target;
 
 	switch (params->transport_type)
 	{
 	case TT_TCP_SSL:
 		if (params->use_tls_sni)
-			target = new RouteTargetSNI(params->hostname);
+			target = new RouteTargetTCPSNI(params->hostname);
 		else
 	case TT_TCP:
 			target = new RouteTargetTCP();
@@ -175,9 +196,12 @@ CommSchedTarget *RouteResultEntry::create_target(const struct RouteParams *param
 	case TT_UDP:
 		target = new RouteTargetUDP();
 		break;
-	case TT_SCTP:
 	case TT_SCTP_SSL:
-		target = new RouteTargetSCTP();
+		if (params->use_tls_sni)
+			target = new RouteTargetSCTPSNI(params->hostname);
+		else
+	case TT_SCTP:
+			target = new RouteTargetSCTP();
 		break;
 	default:
 		errno = EINVAL;
@@ -198,7 +222,7 @@ CommSchedTarget *RouteResultEntry::create_target(const struct RouteParams *param
 int RouteResultEntry::init(const struct RouteParams *params)
 {
 	const struct addrinfo *addr = params->addrinfo;
-	CommSchedTarget *target;
+	RouteManager::RouteTarget *target;
 
 	if (addr == NULL)//0
 	{
@@ -239,8 +263,8 @@ int RouteResultEntry::init(const struct RouteParams *params)
 
 int RouteResultEntry::add_group_targets(const struct RouteParams *params)
 {
+	RouteManager::RouteTarget *target;
 	const struct addrinfo *addr;
-	CommSchedTarget *target;
 
 	for (addr = params->addrinfo; addr; addr = addr->ai_next)
 	{
@@ -299,7 +323,7 @@ void RouteResultEntry::deinit()
 	}
 }
 
-void RouteResultEntry::notify_unavailable(CommSchedTarget *target)
+void RouteResultEntry::notify_unavailable(RouteManager::RouteTarget *target)
 {
 	if (this->targets.size() <= 1)
 		return;
@@ -325,7 +349,7 @@ void RouteResultEntry::notify_unavailable(CommSchedTarget *target)
 	this->nleft--;
 }
 
-void RouteResultEntry::notify_available(CommSchedTarget *target)
+void RouteResultEntry::notify_available(RouteManager::RouteTarget *target)
 {
 	if (this->targets.size() <= 1 || this->nbreak == 0)
 		return;
@@ -401,7 +425,8 @@ static uint64_t __generate_key(enum TransportType type,
 							   const struct addrinfo *addrinfo,
 							   const std::string& other_info,
 							   const struct EndpointParams *ep_params,
-							   const std::string& hostname)
+							   const std::string& hostname,
+							   SSL_CTX *ssl_ctx)
 {
 	const int params[] = {
 		ep_params->address_family, (int)ep_params->max_connections,
@@ -413,8 +438,9 @@ static uint64_t __generate_key(enum TransportType type,
 		buf += other_info;
 
 	buf.append((const char *)params, sizeof params);
-	if (type == TT_TCP_SSL)
+	if (type == TT_TCP_SSL || type == TT_SCTP_SSL)
 	{
+		buf.append((const char *)&ssl_ctx, sizeof (void *));
 		buf.append((const char *)&ep_params->ssl_connect_timeout, sizeof (int));
 		if (ep_params->use_tls_sni)
 		{
@@ -468,11 +494,20 @@ int RouteManager::get(enum TransportType type,
 					  const struct addrinfo *addrinfo,
 					  const std::string& other_info,
 					  const struct EndpointParams *ep_params,
-					  const std::string& hostname,
+					  const std::string& hostname, SSL_CTX *ssl_ctx,
 					  RouteResult& result)
 {
-	uint64_t key = __generate_key(type, addrinfo, other_info,
-								  ep_params, hostname);
+	if (type == TT_TCP_SSL || type == TT_SCTP_SSL)
+	{
+		static SSL_CTX *global_client_ctx = WFGlobal::get_ssl_client_ctx();
+		if (ssl_ctx == NULL)
+			ssl_ctx = global_client_ctx;
+	}
+	else
+		ssl_ctx = NULL;
+
+	uint64_t key = __generate_key(type, addrinfo, other_info, ep_params,
+								  hostname, ssl_ctx);
 	struct rb_node **p = &cache_.rb_node;
 	struct rb_node *parent = NULL;
 	RouteResultEntry *bound = NULL;
@@ -499,17 +534,6 @@ int RouteManager::get(enum TransportType type,
 	}
 	else
 	{
-		int ssl_connect_timeout = 0;
-		SSL_CTX *ssl_ctx = NULL;
-
-		if (type == TT_TCP_SSL || type == TT_SCTP_SSL)
-		{
-			static SSL_CTX *client_ssl_ctx = WFGlobal::get_ssl_client_ctx();
-
-			ssl_ctx = client_ssl_ctx;
-			ssl_connect_timeout = ep_params->ssl_connect_timeout;
-		}
-
 		struct RouteParams params = {
 			.transport_type			=	type,
 			.addrinfo 				=	addrinfo,
@@ -518,18 +542,11 @@ int RouteManager::get(enum TransportType type,
 			.max_connections		=	ep_params->max_connections,
 			.connect_timeout		=	ep_params->connect_timeout,
 			.response_timeout		=	ep_params->response_timeout,
-			.ssl_connect_timeout	=	ssl_connect_timeout,
+			.ssl_connect_timeout	=	ep_params->ssl_connect_timeout,
 			.use_tls_sni			=	ep_params->use_tls_sni,
 			.hostname				=	hostname,
 		};
 
-		if (StringUtil::start_with(other_info, "?maxconn="))
-		{
-			int maxconn = atoi(other_info.c_str() + 9);
-			if (maxconn > 0)
-				params.max_connections = maxconn;
-		}
-
 		entry = new RouteResultEntry;
 		if (entry->init(&params) >= 0)
 		{
@@ -551,12 +568,12 @@ int RouteManager::get(enum TransportType type,
 void RouteManager::notify_unavailable(void *cookie, CommTarget *target)
 {
 	if (cookie && target)
-		((RouteResultEntry *)cookie)->notify_unavailable((CommSchedTarget *)target);
+		((RouteResultEntry *)cookie)->notify_unavailable((RouteTarget *)target);
 }
 
 void RouteManager::notify_available(void *cookie, CommTarget *target)
 {
 	if (cookie && target)
-		((RouteResultEntry *)cookie)->notify_available((CommSchedTarget *)target);
+		((RouteResultEntry *)cookie)->notify_available((RouteTarget *)target);
 }
 
